/* Written by Yong Li, Depeng Liu                                       */
/* Copyright (c) 2016                  	                               */
/* This program is free software: you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
 /* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License    */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

package cn.ac.ios.util;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import automata.FiniteAutomaton;
import cn.ac.ios.automata.Automaton;
import cn.ac.ios.automata.words.ContextWord;
import cn.ac.ios.automata.words.Word;
import cn.ac.ios.learner.fdfa.LearnerFDFA;
import cn.ac.ios.options.Options;
import cn.ac.ios.oracle.Minimizer;
import cn.ac.ios.query.MembershipOracle;
import cn.ac.ios.query.Query;
import cn.ac.ios.query.QuerySimple;
import dk.brics.automaton.State;
import dk.brics.automaton.Transition;
import gnu.trove.map.TIntObjectMap;
import gnu.trove.map.hash.TIntObjectHashMap;

public class BuechiBuilder {

	public static FiniteAutomaton build(LearnerFDFA learnerFDFA, boolean isUnder) {
		// transform it to Rabit form
		return UtilAutomaton.convertToRabitAutomaton(buildNFA(learnerFDFA, true, isUnder));
	}

	protected static dk.brics.automaton.Automaton buildNFA(LearnerFDFA learnerFDFA, boolean epsilon, boolean isUnder) {
		if (Options.verbose)
			learnerFDFA.getHypothesis();
		// L means Leading and P means Progress
		Automaton autL = learnerFDFA.getLeadingAutomaton();
		TIntObjectMap<State> map = new TIntObjectHashMap<>();
		dk.brics.automaton.Automaton dkAutL = UtilAutomaton.convertToDkAutomaton(map, autL);
		for (int stateNr = 0; stateNr < autL.getNumStates(); stateNr++) {
			Automaton autP = learnerFDFA.getProgressAutomaton(stateNr);
			BitSet accStates = autP.getAcceptingStates();
			List<dk.brics.automaton.Automaton> accAuts = new ArrayList<>();
			int stateInitP = autP.getInitialStates().nextSetBit(0);
			for (int accNr = accStates.nextSetBit(0); accNr >= 0; accNr = accStates.nextSetBit(accNr + 1)) {
				dk.brics.automaton.Automaton dkAutP = UtilAutomaton.convertToDkAutomaton(autP, stateInitP, accNr);
				if(Options.simulation) {
					dkAutP = Minimizer.mimimizeNFA(dkAutP);
				}else {
					dkAutP.minimize();
				}
				dk.brics.automaton.Automaton dkAutLOther = UtilAutomaton.convertToDkAutomaton(autL, stateNr, stateNr);
				if(Options.simulation) {
					dkAutLOther = Minimizer.mimimizeNFA(dkAutLOther);
				}else {
					dkAutLOther.minimize();
				}
				
				dk.brics.automaton.Automaton product = dkAutP.intersection(dkAutLOther);
				if(Options.simulation) {
					product = Minimizer.mimimizeNFA(product);
				}else {
					product.minimize();
				}
				
				if (isUnder) {
					dk.brics.automaton.Automaton dkAutNq = UtilAutomaton.convertToDkAutomaton(autP, accNr, accNr);
					if(Options.simulation) {
						dkAutNq = Minimizer.mimimizeNFA(dkAutNq);
					}else {
						dkAutNq.minimize();
					}

					product = product.intersection(dkAutNq);
					if(Options.simulation) {
						product = Minimizer.mimimizeNFA(product);
					}else {
						product.minimize();
					}
				}

				if (! product.getAcceptStates().isEmpty()) {
					assert product.getAcceptStates().size() == 1;
					if(product.getAcceptStates().size() > 1) {
						System.err.println("More than one accepting state...");
						System.exit(-1);
					}
					product = UtilAutomaton.addEpsilon(product);
					accAuts.add(product);
				}
			}

			State u = map.get(stateNr); // make epsilon or Dollar transitions

			if (epsilon) {
				for (dk.brics.automaton.Automaton dkAut : accAuts) {
					State init = dkAut.getInitialState();
					for (Transition t : init.getTransitions())
						u.addTransition(new Transition(t.getMin(), t.getMax(), t.getDest()));
				}
			} else {
				for (dk.brics.automaton.Automaton p : accAuts) {
					u.addTransition(new Transition(ContextWord.getStringDollar().charAt(0), p.getInitialState()));
				}
			}
		}
		return dkAutL;
	}

	// we use lower method to construct LDBA
	public static FiniteAutomaton buildLDBA(LearnerFDFA learnerFDFA) {
		// L means Leading and P means Progress
		Automaton autL = learnerFDFA.getLeadingAutomaton();
		TIntObjectMap<State> map = new TIntObjectHashMap<>();
		dk.brics.automaton.Automaton dkAutL = UtilAutomaton.convertToDkAutomaton(map, autL);
		for (int stateNr = 0; stateNr < autL.getNumStates(); stateNr++) {
			Automaton autP = learnerFDFA.getProgressAutomaton(stateNr);
			BitSet accStates = autP.getAcceptingStates();
			List<dk.brics.automaton.Automaton> accAuts = new ArrayList<>();
			int stateInitP = autP.getInitialStates().nextSetBit(0);
			for (int accNr = accStates.nextSetBit(0); accNr >= 0; accNr = accStates.nextSetBit(accNr + 1)) {
				// Aq,qf
				dk.brics.automaton.Automaton dkAutP = UtilAutomaton.convertToDkAutomaton(autP, stateInitP, accNr);
				dkAutP.minimize();
				// Mq
				dk.brics.automaton.Automaton dkAutLOther = UtilAutomaton.convertToDkAutomaton(autL, stateNr, stateNr);
				dkAutLOther.minimize();
				dk.brics.automaton.Automaton product = dkAutP.intersection(dkAutLOther);
				product.minimize();
				//Aqf,qf
				dk.brics.automaton.Automaton dkAutNq = UtilAutomaton.convertToDkAutomaton(autP, accNr, accNr);
				dkAutNq.minimize();
				// intersection of three automata
				product = product.intersection(dkAutNq);
				product.minimize();

				if (!product.getAcceptStates().isEmpty()) {
					assert product.getAcceptStates().size() == 1;
					//suppose N = L(product), then N+ = N
					LDBABuilder builder = new LDBABuilder(product);
					dk.brics.automaton.Automaton buechi = builder.buildBuechi();
					//buechi.minimize();  // not sure it will be correct to use DFA minimization
					accAuts.add(buechi);
				}
			}

			State u = map.get(stateNr); // make epsilon transitions

			for (dk.brics.automaton.Automaton dkAut : accAuts) {
				State init = dkAut.getInitialState();
				for (Transition t : init.getTransitions())
					u.addTransition(new Transition(t.getMin(), t.getMax(), t.getDest()));
			}
		}

		dkAutL.removeDeadTransitions();
		return UtilAutomaton.convertToRabitAutomaton(dkAutL);
	}

}
