/* Written by Yong Li, Depeng Liu                                       */
/* Copyright (c) 2016                  	                               */
/* This program is free software: you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
 /* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License    */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

package cn.ac.ios.value;

class TypeFraction implements Type {

	private final ContextValue contextValue;
    private ValueFraction known;
    private final Value valueOne = new ValueFraction(this, 1);
    private final Value valueZero = new ValueFraction(this, 0);
    
    TypeFraction(ContextValue context, Value known) {
    	assert context != null;
    	this.contextValue = context;
        assert known == null || known instanceof ValueFraction;
        this.known = (ValueFraction) known;
        if(known != null) {
        	this.known.setImmutable();
        }
    }
    
    TypeFraction(ContextValue context) {
    	this(context, null);
    }
    
	public Value newValue() {
		if(getKnownValue() != null) {
			return getKnownValue().clone();
		}else {
			return new ValueFraction(this);
		}
	}
	
    public ValueFraction newValue(int value) {
    	return new ValueFraction(this, value);
    }
	
	@Override
	public ContextValue getContext() {
		return contextValue;
	}

	@Override
	public Type clone() {
		return new TypeFraction(contextValue, known);
	}

	@Override
	public void setKnownValue(Value value) {	
		assert value != null;
		assert value instanceof ValueFraction;
		known = (ValueFraction)value.clone();
		known.setImmutable();
	}

	@Override
	public Value getKnownValue() {
		return known;
	}
	
    @Override
	public boolean isFraction() {
		return true;
	}
    
	@Override
	public Value getZero() {
		return valueZero;
	}

	@Override
	public Value getOne() {
		return valueOne;
	}
	
	public boolean equals(Object obj) {
		if(!(obj instanceof TypeFraction)) {
			return false;
		}
		return true;
	}
    
    public String toString() {
    	return "fraction";
    }

}
